package com.unis.ynls.dataDictionary.controller;

import com.unis.ynls.dataDictionary.service.DataDictionaryGroupService;
import com.unis.ynls.dataDictionary.utils.ResponseWrapper;
import com.unis.ynls.dataDictionary.vo.DataDictionaryGroupVO;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 *
 * @ClassName: DataDictionaryGroupController
 * @Description: 数据字典组 管理 控制层类
 * @author 杨祖榕
 * @date 2018年11月11日
 * @version v1.0
 */
@RestController
@RequestMapping("/dataDictionaryGroup")
@Slf4j
@Api(value = "数据字典组 控制层类", tags = "数据字典组 控制层类")
public class DataDictionaryGroupController {

    @Autowired
    private DataDictionaryGroupService dataDictionaryGroupService;

    /**
     * 根据id查询数据字典组
     * @param id
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月11日 上午10:50:23
     * @version v1.0
     */
    @ApiOperation(value = "根据id查询数据字典组", notes = "参数：id")
    @GetMapping(value = "/findById/{id}")
    public ResponseWrapper findById(@PathVariable("id") String id) {
        try {
            DataDictionaryGroupVO dataDictionaryGroupVO = dataDictionaryGroupService.findById(id);
            if (ObjectUtils.equals(dataDictionaryGroupVO, null)) {
                return ResponseWrapper.success(300, "数据字典组数据不存在");
            }
            return ResponseWrapper.success(200, "查询成功", dataDictionaryGroupVO);
        } catch (Exception e) {
            log.error("查询数据字典组数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典组数据查询失败");
        }
    }
    /**
     * 根据pid查询数据字典组
     * @param pid
     * @return ResponseWrapper
     * @version v1.0
     */
    @ApiOperation(value = "根据pid查询数据字典组", notes = "参数：id")
    @GetMapping(value = "/findByPid/{pid}")
    public ResponseWrapper findByPid(@PathVariable("pid") String pid) {
        try {
            DataDictionaryGroupVO dataDictionaryGroupVO = dataDictionaryGroupService.findByPid(pid);
            if (ObjectUtils.equals(dataDictionaryGroupVO, null)) {
                return ResponseWrapper.success(300, "数据字典组数据不存在");
            }
            return ResponseWrapper.success(200, "查询成功", dataDictionaryGroupVO);
        } catch (Exception e) {
            log.error("查询数据字典组数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典组数据查询失败");
        }
    }
    /**
     * 根据pid查询数据字典list集合
     * @param pid
     * @return ResponseWrapper
     * @version v1.0
     */
    @ApiOperation(value = "根据pid查询数据字典list集合", notes = "参数：id")
    @GetMapping(value = "/findByPidList/{pid}")
    public ResponseWrapper findByPidList(@PathVariable("pid") String pid) {
        try {
            List<DataDictionaryGroupVO> dataDictionaryGroupVO = dataDictionaryGroupService.findByPidList(pid);
            if (ObjectUtils.equals(dataDictionaryGroupVO, null)) {
                return ResponseWrapper.success(300, "数据字典组数据不存在");
            }
            return ResponseWrapper.success(200, "查询成功", dataDictionaryGroupVO);
        } catch (Exception e) {
            log.error("查询数据字典组数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典组数据查询失败");
        }
    }
    /**
     * 根据id 删除数据字典组数据
     * @param id
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月11日 上午10:50:23
     * @version v1.0
     */
    @ApiOperation(value = "删除数据字典组", notes = "参数：数据字典组ID")
    @DeleteMapping(value = "/delete/{id}")
    public ResponseWrapper delete(@PathVariable("id") String id) {
        try {
            Boolean flag = dataDictionaryGroupService.delete(id);
            if (! flag) {
                return ResponseWrapper.error(300, "参数错误，删除失败");
            }
            return ResponseWrapper.success(200, "数据字典组数据删除成功");
        } catch (Exception e) {
            log.error("删除数据字典组数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "数据字典组数据删除失败");
        }
    }


    /**
     * 保存数据字典组
     * @param dataDictionaryGroupVO
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月11日 上午10:50:23
     * @version v1.0
     */
    @ApiOperation(value = "保存数据字典组", notes = "参数：数据字典组")
    @PostMapping(value = "/save")
    public ResponseWrapper save(@RequestBody DataDictionaryGroupVO dataDictionaryGroupVO) {
        try {
            DataDictionaryGroupVO id = dataDictionaryGroupService.findById(dataDictionaryGroupVO.getId());
            if(id!=null){
                return ResponseWrapper.error("该字典编码已存在");
            }
            DataDictionaryGroupVO groupName = dataDictionaryGroupService.findByGroupName(dataDictionaryGroupVO.getGroupName());
            if(groupName!=null){
                return ResponseWrapper.error("该字典名称已存在");
            }
            boolean flag = dataDictionaryGroupService.save(dataDictionaryGroupVO);
            if (flag) {
                return ResponseWrapper.success(200, "保存成功");
            }
            return ResponseWrapper.error();
        } catch (Exception e) {
            e.printStackTrace();
            log.error("保存数据字典组，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "保存失败");
        }
    }


    /**
     * 更新数据字典组
     * @param dataDictionaryGroupVO
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月10日 下午16:15:23
     * @version v1.0
     */
    @ApiOperation(value = "更新数据字典组信息", notes = "参数：数据字典组")
    @PostMapping(value = "/update")
    public ResponseWrapper update(@RequestBody DataDictionaryGroupVO dataDictionaryGroupVO) {
        try {
            DataDictionaryGroupVO groupName = dataDictionaryGroupService.findByGroupName(dataDictionaryGroupVO.getGroupName());
            if(groupName!=null){
                return ResponseWrapper.error("该字典名称已存在");
            }
            boolean flag = dataDictionaryGroupService.update(dataDictionaryGroupVO);
            if (flag) {
                return ResponseWrapper.success(200,"修改成功");
            }
            return ResponseWrapper.error();
        } catch (Exception e) {
            e.printStackTrace();
            log.error("更新数据字典组数据，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "操作失败");
        }
    }



    /**
     * 查询所有的数据字典组
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月11日 下午16:15:23
     * @version v1.0
     */
    @ApiOperation(value = "查询所有的数据字典组", notes = "查询所有的数据字典组")
    @GetMapping(value = "/getAll")
    public ResponseWrapper getAll(@ApiParam(value="字典组名称", required = false) @RequestParam(value = "groupName",required = false)String groupName) {
        try {
            List<DataDictionaryGroupVO> list = dataDictionaryGroupService.getAll(groupName);
            return ResponseWrapper.success( 200, "查询成功", list );
        } catch (Exception e) {
            log.error("数据字典组 查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "数据字典组 查询失败");
        }
    }

    /**
     * 查询所有的数据字典组
     * @return ResponseWrapper
     * @author 杨祖榕
     * @date 2018年11月11日 下午16:15:23
     * @version v1.0
     */
    @ApiOperation(value = "查询所有的数据字典组分页", notes = "查询所有的数据字典组")
    @ApiImplicitParams({ @ApiImplicitParam(name = "page", value = "起始页", required = true),
            @ApiImplicitParam(name = "size", value = "每页展示行数", required = true) })
    @GetMapping(value = "/getAllAndPageUp/{page}/{size}")
    public ResponseWrapper getAllAndPageUp(@ApiParam(value="字典组名称", required = false) @RequestParam(value = "groupName",required = false)String groupName, @PathVariable Integer page,
                                           @PathVariable Integer size) {
        try {
            Map<String, Object> list = dataDictionaryGroupService.getAllPage(groupName, page, size);
            return ResponseWrapper.success( 200, "查询成功", list );
        } catch (Exception e) {
            log.error("数据字典组 查询失败{}", e.getMessage());
            return ResponseWrapper.error(500, "数据字典组 查询失败");
        }
    }

}
