package com.unis.ynls.dataDictionary.service.impl;


import com.unis.common.utils.CurrentUtil;
import com.unis.common.utils.DateUtil;
import com.unis.common.utils.LsBeanUtils;
import com.unis.ynls.dataDictionary.entity.DataDictionaryGroupDO;
import com.unis.ynls.dataDictionary.mapper.DataDictionaryGroupMapper;
import com.unis.ynls.dataDictionary.service.DataDictionaryGroupService;
import com.unis.ynls.dataDictionary.vo.DataDictionaryGroupVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: DataDictionaryGroupServiceImpl
 * @Description: 数据字典组管理
 * @author 杨祖榕
 * @date 2018年11月09日 19:59:23
 * @version v1.0
 */
@Service
@Slf4j
public class DataDictionaryGroupServiceImpl implements DataDictionaryGroupService {

    @Autowired
    private DataDictionaryGroupMapper dataDictionaryGroupMapper;
/*    @Autowired
    private BaseRepository baseRepository;*/
//    @Autowired
//    private CurrentUtils currentUtils;

    /**
     * 根据id查询数据字典组
     * @Author 杨祖榕
     * @Date 2018年11月10日 11:23:23
     * @param id
     * @return
     */
    @Override
    public DataDictionaryGroupVO findById(String id) {
        DataDictionaryGroupDO dataDictionaryGroupDO = dataDictionaryGroupMapper.selectByPrimaryKey(id);
        DataDictionaryGroupVO dataDictionaryGroupVO = LsBeanUtils.copyProperties(dataDictionaryGroupDO, DataDictionaryGroupVO.class);
        return dataDictionaryGroupVO;
    }

    @Override
    public DataDictionaryGroupVO findByGroupName(String groupNmae) {
        DataDictionaryGroupDO dataDictionaryGroupDO = dataDictionaryGroupMapper.selectByName(groupNmae);
        DataDictionaryGroupVO dataDictionaryGroupVO = LsBeanUtils.copyProperties(dataDictionaryGroupDO, DataDictionaryGroupVO.class);
        return dataDictionaryGroupVO;
    }

    @Override
    @Transactional
    public Boolean delete(String id) {
        int count = dataDictionaryGroupMapper.deleteByPrimaryKey(id);
        return count > 0 ? true : false;
    }

    @Override
    @Transactional
    public boolean save(DataDictionaryGroupVO dataDictionaryGroupVO) {

        DataDictionaryGroupDO dataDictionaryGroupDO = new DataDictionaryGroupDO();
        LsBeanUtils.copyPropertiesIgnoreNull(dataDictionaryGroupVO, dataDictionaryGroupDO);
        dataDictionaryGroupDO.setCreateBy(CurrentUtil.getUserName());
        dataDictionaryGroupDO.setCreateTime(DateUtil.getNowDate());
        int insert = dataDictionaryGroupMapper.insertSelective(dataDictionaryGroupDO);
        return insert > 0 ? true : false;
    }

    @Override
    @Transactional
    public boolean update(DataDictionaryGroupVO dataDictionaryGroupVO) {
        DataDictionaryGroupDO dataDictionaryGroupDO = dataDictionaryGroupMapper.selectByPrimaryKey(dataDictionaryGroupVO.getId());
        BeanUtils.copyProperties(dataDictionaryGroupVO, dataDictionaryGroupDO);

        dataDictionaryGroupDO.setUpdateBy(CurrentUtil.getUserName());
        dataDictionaryGroupDO.setUpdateTime(DateUtil.getNowDate());

        int count = dataDictionaryGroupMapper.updateByPrimaryKey(dataDictionaryGroupDO);
        return count > 0 ? true : false;
    }

    @Override
    public List<DataDictionaryGroupVO> getAll(String groupName) {
        List<DataDictionaryGroupVO> result = null;

        Example example = new Example(DataDictionaryGroupDO.class);
        Example.Criteria criteria = example.createCriteria();
        //criteria.andIsNull("deleteBy");
        criteria.andCondition("delete_by = '' or delete_by is null");
        if(StringUtils.isNotEmpty(groupName)) {
            criteria.andLike("groupName", "%" + groupName + "%");
        }

        example.setOrderByClause("ORDER_BY asc");

        List<DataDictionaryGroupDO> list = dataDictionaryGroupMapper.selectByExample(example);
        if(null != list && list.size() > 0){
            result = new ArrayList<>();
            for(DataDictionaryGroupDO dataDictionaryGroupDO : list) {
                DataDictionaryGroupVO dataDictionaryGroupVO = LsBeanUtils.copyProperties(dataDictionaryGroupDO, DataDictionaryGroupVO.class);
                result.add(dataDictionaryGroupVO);
            }
        }

        return result;

    }

    @Override
    public DataDictionaryGroupVO findByPid(String pid) {
        DataDictionaryGroupDO dataDictionaryGroupDO = dataDictionaryGroupMapper.findByPid(pid);
        DataDictionaryGroupVO dataDictionaryGroupVO = LsBeanUtils.copyProperties(dataDictionaryGroupDO, DataDictionaryGroupVO.class);
        return dataDictionaryGroupVO;
    }

    @Override
    public List<DataDictionaryGroupVO> findByPidList(String pid) {
        List<DataDictionaryGroupVO> result = null;
        List<DataDictionaryGroupDO> list =  dataDictionaryGroupMapper.findByPidList(pid);
        if(null != list && list.size() > 0){
            result = new ArrayList<>();
            for(DataDictionaryGroupDO dataDictionaryGroupDO : list) {
                DataDictionaryGroupVO dataDictionaryGroupVO = LsBeanUtils.copyProperties(dataDictionaryGroupDO, DataDictionaryGroupVO.class);
                result.add(dataDictionaryGroupVO);
            }
        }
        return result;
    }

    @Override
    public Map<String, Object> getAllPage(String groupName, int page, int size) {
        List<DataDictionaryGroupVO> result = null;

        Example example = new Example(DataDictionaryGroupDO.class);
        Example.Criteria criteria = example.createCriteria();
//        criteria.andIsNull("deleteBy");
        //criteria.andCondition("deleteBy = ", "");
        criteria.andCondition("delete_by = '' or delete_by is null");
        if(StringUtils.isNotEmpty(groupName)) {
            criteria.andLike("groupName", "%" + groupName + "%");
        }

        example.setOrderByClause("create_time desc");
        Map<String, Object> resMap = new HashMap<>();

        List<DataDictionaryGroupDO> list = dataDictionaryGroupMapper.selectByExample(example);
        if(null != list && list.size() > 0){
            result = new ArrayList<>();
            for(DataDictionaryGroupDO dataDictionaryGroupDO : list) {
                DataDictionaryGroupVO dataDictionaryGroupVO = LsBeanUtils.copyProperties(dataDictionaryGroupDO, DataDictionaryGroupVO.class);
                result.add(dataDictionaryGroupVO);
            }
            resMap.put("total", result == null ? 0: result.size());
            // 从第几条数据开始
            int firstIndex = (page - 1) * size;
            // 到第几条数据结束
            int lastIndex = page * size;
            if (result.size() >= lastIndex) {
                result = result.subList(firstIndex, lastIndex);
            } else {
                result = result.subList(firstIndex, result.size());
            }
        }
        resMap.put("resultList", result);

        return resMap;
    }

}
