package com.unis.ynls.dataDictionary.service.impl;

import com.unis.common.utils.CurrentUtil;
import com.unis.common.utils.DateUtil;
import com.unis.common.utils.LsBeanUtils;
import com.unis.ynls.dataDictionary.entity.DataDictionaryDO;
import com.unis.ynls.dataDictionary.mapper.DataDictionaryMapper;
import com.unis.ynls.dataDictionary.service.DataDictionaryService;
import com.unis.ynls.dataDictionary.utils.RedisUtil;
import com.unis.ynls.dataDictionary.vo.DataDictionaryVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName: DataDictionaryServiceImpl
 * @Description: 数据字典管理
 * @author 杨祖榕
 * @date 2018年11月09日 19:59:23
 * @version v1.0
 */
@Service
@Slf4j
public class DataDictionaryServiceImpl implements DataDictionaryService {

    private final String GROUP_REDIS_KEY_PRE = "GROUP_DATA_DICTIONARY@";
    private final String SINGLE_REDIS_KEY_PRE = "SINGLE_DATA_DICTIONARY@";
    private final String MID_AT = "@";

    @Autowired
    private DataDictionaryMapper dataDictionaryMapper;

    @Autowired
    private RedisUtil<DataDictionaryVO> redisUtil;

//    @Autowired
//    private CurrentUtils currentUtils;


    /**
     * 根据id查询数据字典
     * @Author 杨祖榕
     * @Date 2018年11月10日 11:23:23
     * @param id
     * @return
     */
    @Override
    public DataDictionaryVO findById(Integer id) {
        DataDictionaryDO dataDictionaryDO = dataDictionaryMapper.selectByPrimaryKey(id);
        DataDictionaryVO dataDictionaryVO = LsBeanUtils.copyProperties(dataDictionaryDO, DataDictionaryVO.class);
        return dataDictionaryVO;
    }

    @Override
    public DataDictionaryVO getDataDictionaryByGroupIdAndDictCode(String groupId, String dictCode) {

        DataDictionaryVO result = null;

        String redisKey = SINGLE_REDIS_KEY_PRE + groupId + MID_AT + dictCode;

        result = redisUtil.getObjectByJson(redisKey, DataDictionaryVO.class);
        if(null != result) {
            return result;
        }

        Example example = new Example(DataDictionaryDO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andIsNull("deleteBy");
        criteria.andEqualTo("groupId", groupId);
        criteria.andEqualTo("dictCode", dictCode);

        DataDictionaryDO dataDictionaryDO = dataDictionaryMapper.selectOneByExample(example);
        if(null != dataDictionaryDO) {
            result = LsBeanUtils.copyProperties(dataDictionaryDO, DataDictionaryVO.class);
            redisUtil.setObjectByJson(redisKey, result);
        }
        return result;
    }

    @Override
    @Transactional
    public Boolean delete(Integer id) {
        DataDictionaryVO VO = findById(id);
        int count = dataDictionaryMapper.deleteByPrimaryKey(id);
        Boolean result = count > 0 ? true : false;
        if(result) {
            //删除整组数据字典
            redisUtil.delete(GROUP_REDIS_KEY_PRE + VO.getGroupId());
            //删除数据字典
            redisUtil.delete(SINGLE_REDIS_KEY_PRE + VO.getGroupId() + MID_AT + VO.getDictCode());
        }
        return result;
    }

    @Override
    @Transactional
    public boolean save(DataDictionaryVO dataDictionaryVO) {

        DataDictionaryDO dataDictionaryDO = new DataDictionaryDO();
        Integer maxId = dataDictionaryMapper.getMaxId();
        maxId = (maxId == null ? 1 : maxId + 1);
        dataDictionaryDO.setId(maxId);

        LsBeanUtils.copyPropertiesIgnoreNull(dataDictionaryVO, dataDictionaryDO);
        dataDictionaryDO.setCreateBy(CurrentUtil.getUserName());
        dataDictionaryDO.setCreateTime(DateUtil.getNowDate());

        int insert = dataDictionaryMapper.insert(dataDictionaryDO);
        //删除整组数据字典
        redisUtil.delete(GROUP_REDIS_KEY_PRE + dataDictionaryDO.getGroupId());
        //删除数据字典
        redisUtil.delete(SINGLE_REDIS_KEY_PRE + dataDictionaryDO.getGroupId() + MID_AT + dataDictionaryDO.getDictCode());

        return insert > 0 ? true : false;
    }

    @Override
    @Transactional
    public boolean update(DataDictionaryVO dataDictionaryVO) {
        int count = 0;
        DataDictionaryDO dataDictionaryDO = dataDictionaryMapper.selectByPrimaryKey(dataDictionaryVO.getId());
        if(null != dataDictionaryDO) {
            BeanUtils.copyProperties(dataDictionaryVO, dataDictionaryDO);

            dataDictionaryDO.setUpdateBy(CurrentUtil.getUserName());
            dataDictionaryDO.setUpdateTime(DateUtil.getNowDate());

            count = dataDictionaryMapper.updateByPrimaryKey(dataDictionaryDO);
            //删除整组数据字典
            redisUtil.delete(GROUP_REDIS_KEY_PRE + dataDictionaryDO.getGroupId());
            //删除数据字典
            redisUtil.delete(SINGLE_REDIS_KEY_PRE + dataDictionaryDO.getGroupId() + MID_AT + dataDictionaryDO.getDictCode());
        }
        return count > 0 ? true : false;
    }

    @Override
    public List<DataDictionaryVO> getSubDataDictionarysByDictCodeId(String dictCode) {
        List<DataDictionaryVO> result = null;

        Example example = new Example(DataDictionaryDO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andIsNull("deleteBy");
        criteria.andEqualTo("pdictCode", dictCode);
        example.setOrderByClause("ORDER_BY asc");

        List<DataDictionaryDO> list = dataDictionaryMapper.selectByExample(example);

        result = dataDictionaryDOList2dataDictionaryVOList(list, result);

        return result;
    }

    @Override
    public List<DataDictionaryVO> getSubDataDictionarysByGroupId(String groupId) {
        List<DataDictionaryVO> result = new ArrayList<DataDictionaryVO>();

        //tring pdictCode = "0";
        Example example = new Example(DataDictionaryDO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andIsNull("deleteBy");
        //criteria.andEqualTo("pdictCode", pdictCode);
        //criteria.andCondition("pdictCode = '0' or pdictCode = '101'");
        criteria.andEqualTo("groupId", groupId);
        example.setOrderByClause("ORDER_BY desc");

        List<DataDictionaryDO> list = dataDictionaryMapper.selectByExample(example);

        result = dataDictionaryDOList2dataDictionaryVOList(list, result);

        return result;
    }

    @Override
    public List<DataDictionaryVO> getDataTree(String groupId, String dictCode) {
        DataDictionaryVO dataTree = getDataTreeByDictCode(groupId, dictCode);
        return dataTree == null ? null : dataTree.getSubDataDictionarys();
    }

    @Override
    public List<DataDictionaryVO> getDataTreeByGroupId(String groupId) {
        String redisKey = null;
        List<DataDictionaryVO> result = null;
        System.out.println("redis返回数据++++++++++++++++++++++++++++++++++++++"+result);
//        List<DataDictionaryVO> result=new ArrayList();
        if(null == result || result.size() <= 0) {
            result = getDataDictionaryTreeByGroupIdResolveBug(groupId);
            //
           /* if(null != result && result.size() > 0) {
                redisUtil.setListDataByJson(redisKey, result);
            }*/
        }

        if(result!=null && result.size()>0 && result.get(0) != null){
            if(result.get(0).getSubDataDictionarys() != null && result.get(0).getSubDataDictionarys().size()>0){
                Collections.sort(result.get(0).getSubDataDictionarys(),new Comparator<DataDictionaryVO>(){
                    public int compare(DataDictionaryVO arg0, DataDictionaryVO arg1) {
                        return arg0.getId().compareTo(arg1.getId());
                    }
                });
            }
        }

        return result;
    }



    @Override
    public List<DataDictionaryVO> getDataGroupId(String groupId) {
        String redisKey = "DataDictionaryVO" + groupId;
        List<DataDictionaryVO> result = redisUtil.getListDataByJson(redisKey, DataDictionaryVO.class);
        if(null == result || result.size() <= 0) {
            result = getSubDataDictionarysByGroupId(groupId);
        }
        return result;
    }


    /**
     * 获取某ID下的数据字典树，以该ID的数据字典为根节点
     * @param groupId
     * @param dictCode
     * @return
     */
    public DataDictionaryVO getDataTreeByDictCode(String groupId, String dictCode) {
        DataDictionaryVO result = null;

        List<DataDictionaryVO> tempVoList = null;
        List<DataDictionaryDO> list = dataDictionaryMapper.getDataTree(groupId, dictCode);
        //去掉 List集合去除重复数据
        for(int i=0;i<list.size();i++){
            for (int j  = list.size()- 1;j>i; j -- )  {
                if  (list.get(j).getDictCode().equals(list.get(i).getDictCode())){
                    list.remove(j);
                }
            }
        }
        if(list.size()>0){
            Collections.sort(list,new Comparator<DataDictionaryDO>(){
                public int compare(DataDictionaryDO arg0, DataDictionaryDO arg1) {
                    return arg0.getId().compareTo(arg1.getId());
                }
            });
        }

        tempVoList = dataDictionaryDOList2dataDictionaryVOList(list, tempVoList);

        // 处理层级关系
        if(null != tempVoList && tempVoList.size() > 0){
            DataDictionaryVO dataDictionaryVO = null;//根节点
            for(DataDictionaryVO tempVO : tempVoList){
                if(dictCode.equals(tempVO.getDictCode())){
                    dataDictionaryVO = tempVO;
                    break;
                }
            }
            if(null != dataDictionaryVO){//以该根节点为起始点，递归寻找子节点
                dataDictionaryLayerRecursive(tempVoList, dataDictionaryVO);
                result = dataDictionaryVO;
            }
        }
        return result;
    }

    private List<DataDictionaryVO> dataDictionaryDOList2dataDictionaryVOList(List<DataDictionaryDO> doList, List<DataDictionaryVO> voList){
        if(null != doList && doList.size() > 0){
            voList = new ArrayList<>();
            for(DataDictionaryDO dataDictionaryDO : doList){
                DataDictionaryVO dataDictionaryVO = LsBeanUtils.copyProperties(dataDictionaryDO, DataDictionaryVO.class);
                voList.add(dataDictionaryVO);
            }
        }

        return voList;
    }



    /**
     * 递归处理 数据字典 的层级关级
     * @param list
     * @param dataDictionaryVO
     */
    private void dataDictionaryLayerRecursive(List<DataDictionaryVO> list, DataDictionaryVO dataDictionaryVO){
        if(null != list && list.size() > 0){
            //1、获取 dataDictionaryVO 的下一层级所有的数据字典
            for(DataDictionaryVO tempVO : list) {
                if(tempVO.getPdictCode().equals(dataDictionaryVO.getDictCode())){
                    List<DataDictionaryVO> subDataDictionarys = dataDictionaryVO.getSubDataDictionarys();
                    if(null == subDataDictionarys) {
                        subDataDictionarys = new ArrayList<>();
                    }
                    subDataDictionarys.add(tempVO);
                    dataDictionaryVO.setSubDataDictionarys(subDataDictionarys);
                }
            }

            list.remove(dataDictionaryVO);

            //2、对下一层级的各数据字典遍历，对子层的每个数据字典再获取下一层级所有的数据字典
            if(list.size() > 0){
                List<DataDictionaryVO> subDataDictionarys = dataDictionaryVO.getSubDataDictionarys();
                if(null != subDataDictionarys && subDataDictionarys.size() > 0){
                    for(DataDictionaryVO tempVO : subDataDictionarys) {
                        dataDictionaryLayerRecursive(list, tempVO);
                    }
                }
            }
        }
    }

    //LPH
    private List<DataDictionaryVO> getDataDictionaryTreeByGroupIdResolveBug(String groupId) {

        //String redisKey = GROUP_REDIS_KEY_PRE + groupId;
        // List<DataDictionaryVO> dataDictionaryList = redisUtil.getListDataByJson(redisKey, DataDictionaryVO.class);
        List<DataDictionaryVO> dataDictionaryList = getSubDataDictionarysByGroupId(groupId);
        if(null != dataDictionaryList && dataDictionaryList.size() > 0){
            Map<String,List<DataDictionaryVO>> map=new HashMap<>();
            for(DataDictionaryVO tempVO : dataDictionaryList){
                String pCode=tempVO.getPdictCode();
                if(map.containsKey(pCode)){
                    map.get(pCode).add(tempVO);
                }else{
                    List<DataDictionaryVO> list=new ArrayList<>();
                    list.add(tempVO);
                    map.put(pCode, list);
                }
            }
            String baseId1="-1",baseId2="0";
            Iterator<DataDictionaryVO> it=dataDictionaryList.iterator();
            String id=null,parentId=null;
            DataDictionaryVO dictionaryVO=null;
            while(it.hasNext()){
                dictionaryVO=it.next();
                id=dictionaryVO.getDictCode();
                parentId=dictionaryVO.getPdictCode();
                if(map.containsKey(id)){
                    dictionaryVO.setSubDataDictionarys(map.get(id));
                }
                if(!baseId1.equals(parentId) && !baseId2.equals(parentId)  ){
                    it.remove();
                }
            }
        }
        return dataDictionaryList;
    }
    private List<DataDictionaryVO> getDataDictionaryTreeByGroupId(String groupId) {

        //String redisKey = GROUP_REDIS_KEY_PRE + groupId;
       // List<DataDictionaryVO> dataDictionaryList = redisUtil.getListDataByJson(redisKey, DataDictionaryVO.class);
        List<DataDictionaryVO> result1 = null;
        List<DataDictionaryVO> dataDictionaryList = getSubDataDictionarysByGroupId(groupId);
        if(null != dataDictionaryList && dataDictionaryList.size() > 0){
            result1 = new ArrayList<>();
            for(DataDictionaryVO tempVO : dataDictionaryList){
                DataDictionaryVO dataTree = getDataTreeByDictCode(groupId, tempVO.getDictCode());
                result1.add(dataTree);
            }
        }
        return result1;
    }

//    @Override
//    public void clearRedisDataDictionary() {
//        String keyOne = "keyOne";
//
//        String keyTwo = "keyTwo";
//
//        DataDictionaryVO vo = new DataDictionaryVO();
//        vo.setId(1);
//        vo.setDictName("你好！");
//
//        DataDictionaryVO vo1 = new DataDictionaryVO();
//        vo1.setId(2);
//        vo1.setDictName("我好！");
//
//
//        /////////////////////////////////////////////////
//        Set<DataDictionaryVO> set = new HashSet<>();
//        set.add(vo);
//        set.add(vo1);
//
//        redisUtil.setSetDataByJson(redisTemplate, keyOne, set);
//
//        Set<DataDictionaryVO> getSet = redisUtil.getSetDataByJson(redisTemplate, keyOne, DataDictionaryVO.class);
//        for(DataDictionaryVO vvo : getSet) {
//            System.out.println(vvo.getId() + "：" + vvo);
//        }
//
//
//        System.out.println("================================================================================================");
//
//        Map<String, DataDictionaryVO> map = new HashMap<>();
//        map.put("aaa", vo);
//        map.put("bbb", vo1);
//        redisUtil.setMapDataByJson(redisTemplate, keyTwo, map);
//
//        Map<String, DataDictionaryVO> getMap = redisUtil.getMapDataByJson(redisTemplate, keyTwo, DataDictionaryVO.class);
//        Set<String> keySet = getMap.keySet();
//        for(String tKey : keySet ) {
//            DataDictionaryVO value = getMap.get(tKey);
//            System.out.println(tKey + "：" + value.getId() + "：" + value);
//        }
//    }

    @Override
    public void dataDictionarys2Redis(){

        //查询出所有的数据字典，必须优先按组排序，这样可以进行分组(分类)处理
        List<DataDictionaryDO> doList = dataDictionaryMapper.getAllOrderbyGroup();
        if(null != doList && doList.size() > 0){
            List<DataDictionaryVO> voList = dataDictionaryDOList2dataDictionaryVOList(doList, null);

            List<DataDictionaryVO> toRedisData = new ArrayList<>();
            DataDictionaryVO tempVO = voList.get(0);
           if (tempVO!=null){
                for(DataDictionaryVO dataDictionaryVO : voList) {
                    String groupId = dataDictionaryVO.getGroupId();
                    String dictCode = dataDictionaryVO.getDictCode();
                    //单个数据字典存入Redis
                    redisUtil.setObjectByJson(SINGLE_REDIS_KEY_PRE + groupId + MID_AT + dictCode, dataDictionaryVO);
                    if(tempVO.getGroupId()!=null){
                        if (tempVO.getGroupId().equals(dataDictionaryVO.getGroupId())) {
                            toRedisData.add(dataDictionaryVO);
                        } else {
                            //处理 toRedisData ,将toRedisData 进行改造成 一组数据字典，并存入Redis
                            packageDataDictionaryListToRedisData(tempVO.getGroupId(), toRedisData);
                            //清空toRedisData
                            toRedisData.clear();
                            //改变 tempVO 的值，处理下一批 数据字典
                            tempVO = dataDictionaryVO;
                        }
                    }
                }
            }

            if(null != toRedisData && toRedisData.size() >0 ) {
                packageDataDictionaryListToRedisData(tempVO.getGroupId(), toRedisData);
                //清空toRedisData
                toRedisData.clear();
            }
        }
    }

    @Override
    public List<DataDictionaryDO> findByGroupId(String groupId) {
       return dataDictionaryMapper.findByGroupId(groupId);
    }



    private void packageDataDictionaryListToRedisData(String groupId, List<DataDictionaryVO> list){
        List<DataDictionaryVO> result = new ArrayList<>();

        int size = list.size();

        List<DataDictionaryVO> pDictCode_0_voList = new ArrayList<>();
        for(DataDictionaryVO VO : list){
            if(VO.getPdictCode().equals("0")) {
                pDictCode_0_voList.add(VO);
            }
        }

        if(pDictCode_0_voList.size() == size) {//说明该批数据字典组，每个即是根节点，也是叶子节点
            result = list;
        } else {
            list.removeAll(pDictCode_0_voList);
            for(DataDictionaryVO VO : pDictCode_0_voList) {
                VO = packageDataDictionary(VO, list);
                result.add(VO);
            }
        }

        String redisKey = GROUP_REDIS_KEY_PRE + groupId;
        redisUtil.setListDataByJson(redisKey, result);
    }


    private DataDictionaryVO packageDataDictionary(DataDictionaryVO PVO, List<DataDictionaryVO> list) {

        for(DataDictionaryVO VO : list) {
            if(VO.getPdictCode().equals(PVO.getDictCode())) {
                List<DataDictionaryVO> subDataDictionarys = PVO.getSubDataDictionarys();
                if(null == subDataDictionarys) {
                    subDataDictionarys = new ArrayList<>();
                }
                subDataDictionarys.add(VO);
                PVO.setSubDataDictionarys( subDataDictionarys );
            }
        }

        if(null != list && list.size() > 0) {
            List<DataDictionaryVO> subDataDictionarys = PVO.getSubDataDictionarys();
            if(null != subDataDictionarys && subDataDictionarys.size() > 0) {
                list.removeAll(subDataDictionarys);
                //
                for(DataDictionaryVO subDataDictionary : subDataDictionarys) {
                    packageDataDictionary(subDataDictionary, list);
                }
            }
        }

        return PVO;
    }

    
    @Override
	public List<DataDictionaryVO> getSubDataDictionarys(String groupId , String pdictCode) {
		Example example = new Example(DataDictionaryDO.class);
		Example.Criteria criteria = example.createCriteria();
		criteria.andIsNull("deleteBy");
		criteria.andEqualTo("pdictCode", pdictCode);
		criteria.andEqualTo("groupId", groupId);
		example.orderBy("groupId").asc();
		List<DataDictionaryDO> list = dataDictionaryMapper.selectByExample(example);
		List<DataDictionaryVO> voList = new ArrayList<>();
		if (null != list && list.size() > 0) {
			for (DataDictionaryDO dataDictionaryDO : list) {
				DataDictionaryVO dataDictionaryVO = LsBeanUtils.copyProperties(dataDictionaryDO,DataDictionaryVO.class);
				List<DataDictionaryVO> subDataDictionarys1 = getSubDataDictionarys(dataDictionaryVO.getGroupId(),dataDictionaryVO.getDictCode());
                if(!subDataDictionarys1.isEmpty()) {
                    dataDictionaryVO.setSubDataDictionarys(subDataDictionarys1);
                }
				voList.add(dataDictionaryVO);
			}
		}
		return voList;
	}
    /**
     * 根据数据字典组ID和数据字典名称查询数据字典
     * @param groupId
     * @param dictName
     * @return DataDictionaryVO
     * @author liyb
     * @date 2019-03-19 11:02:10
     * @version v1.0
     */
    @Override
    public String getDataDictionaryByGroupIdAndDictName(String groupId, String dictName) {
        Example example = new Example(DataDictionaryDO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andIsNull("deleteBy");
        criteria.andEqualTo("groupId", groupId);
        criteria.andEqualTo("dictName", dictName);
        DataDictionaryDO dataDictionaryDO = dataDictionaryMapper.selectOneByExample(example);
        if(ObjectUtils.notEqual(dataDictionaryDO,null)) {
            return dataDictionaryDO.getDictCode();
        }else{
            return null;
        }
    }

    public List<DataDictionaryVO> getSubDataDictionarys1(String id) {
		Example example = new Example(DataDictionaryDO.class);
		Example.Criteria criteria = example.createCriteria();
		criteria.andIsNull("deleteBy");
		criteria.andEqualTo("pdictCode", id);
		List<DataDictionaryDO> dataDictionaryDoList = dataDictionaryMapper.selectByExample(example);
		List<DataDictionaryVO> list = new ArrayList<>();
		for (DataDictionaryDO dataDictionaryDO2 : dataDictionaryDoList) {
			DataDictionaryVO dataDictionaryVO = LsBeanUtils.copyProperties(dataDictionaryDO2, DataDictionaryVO.class);
			list.add(dataDictionaryVO);
		}
		return list;

	}


	@Override
	public DataDictionaryVO getDictCode(String groupId, String dictCode) {
		DataDictionaryVO dataDictionaryVO = getDataDictionaryByGroupIdAndDictCode(groupId, dictCode);
		DataDictionaryVO dataVO = getDictCode(dataDictionaryVO.getPdictCode());
		return dataVO;
	}

    @Override
    public List<DataDictionaryDO> findByPDictCode(String pdictCode) {
        return dataDictionaryMapper.findByPDictCode(pdictCode);
    }

    @Override
    public String getDataDictionaryByGroupIdAndInDictName(String groupId, String dictName) {
        List<String> list =  dataDictionaryMapper.getDataDictionaryByGroupIdAndInDictName(groupId, dictName);
        if(null != list && list.size() > 0) {
            return list.stream().map(String::valueOf).collect(Collectors.joining(","));
        } else {
            return null;
        }
    }

    private DataDictionaryVO getDictCode(String id) {
		Example example = new Example(DataDictionaryDO.class);
		Example.Criteria criteria = example.createCriteria();
		String groupId = "AREA_CODE";
		criteria.andIsNull("deleteBy");
		criteria.andEqualTo("dictCode", id);
		criteria.andEqualTo("groupId", groupId);
		DataDictionaryDO dataDictionaryDO = dataDictionaryMapper.selectOneByExample(example);
		DataDictionaryVO dataDictionaryVO = LsBeanUtils.copyProperties(dataDictionaryDO, DataDictionaryVO.class);
		return dataDictionaryVO;

	}

    @Override
    public List<DataDictionaryDO> getDataParent(String groupId, String dictCode) {
        List<DataDictionaryDO> list = dataDictionaryMapper.getDataParent(groupId,dictCode);
        return list;
    }
}
